@@ -65,6 +65,7 @@ induct name] }");
 	PUTS("\tq\tQuit");
 	PUTS("\th\tHelp");
 	PUTS("\t?\tHelp");
+	PUTS("\tv\tPrint version of ION.");
 	PUTS("\ta\tAdd");
 	PUTS("\t   a plan <node name> <default directive>");
 	PUTS("\t   a rule <node name> <demux name> <directive>");
@@ -105,7 +106,7 @@ static int	parseDirective(char *actionToken, char *parmToken,
 		}
 
 		dir->action = fwd;
-		sdr_begin_xn(sdr);
+		CHKZERO(sdr_begin_xn(sdr));
 		dir->eid = sdr_string_create(sdr, parmToken);
 		if (sdr_end_xn(sdr))
 		{
@@ -159,7 +160,7 @@ static int	parseDirective(char *actionToken, char *parmToken,
 		dir->outductElt = vduct->outductElt;
 		if (destDuctName)
 		{
-			sdr_begin_xn(sdr);
+			CHKZERO(sdr_begin_xn(sdr));
 			dir->destDuctName = sdr_string_create(sdr,
 					destDuctName);
 			if (sdr_end_xn(sdr))
@@ -396,6 +397,7 @@ static void	printPlan(Dtn2Plan *plan)
 
 static void	infoPlan(int tokenCount, char **tokens)
 {
+	Sdr	sdr = getIonsdr();
 	Object	planAddr;
 		OBJ_POINTER(Dtn2Plan, plan);
 	Object	elt;
@@ -406,15 +408,19 @@ static void	infoPlan(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	dtn2_findPlan(tokens[2], &planAddr, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown plan.");
-		return;
+	}
+	else
+	{
+		GET_OBJ_POINTER(sdr, Dtn2Plan, plan, planAddr);
+		printPlan(plan);
 	}
 
-	GET_OBJ_POINTER(getIonsdr(), Dtn2Plan, plan, planAddr);
-	printPlan(plan);
+	sdr_exit_xn(sdr);
 }
 
 static void	printRule(Dtn2Plan *plan, Dtn2Rule *rule)
@@ -454,23 +460,28 @@ static void	infoRule(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	dtn2_findPlan(tokens[2], &planAddr, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown plan.");
-		return;
 	}
-
-	GET_OBJ_POINTER(sdr, Dtn2Plan, plan, planAddr);
-	dtn2_findRule(tokens[2], tokens[3], plan, &ruleAddr, &elt);
-	if (elt == 0)
+	else
 	{
-		printText("Unknown rule.");
-		return;
+		GET_OBJ_POINTER(sdr, Dtn2Plan, plan, planAddr);
+		dtn2_findRule(tokens[2], tokens[3], plan, &ruleAddr, &elt);
+		if (elt == 0)
+		{
+			printText("Unknown rule.");
+		}
+		else
+		{
+			GET_OBJ_POINTER(sdr, Dtn2Rule, rule, ruleAddr);
+			printRule(plan, rule);
+		}
 	}
 
-	GET_OBJ_POINTER(sdr, Dtn2Rule, rule, ruleAddr);
-	printRule(plan, rule);
+	sdr_exit_xn(sdr);
 }
 
 static void	executeInfo(int tokenCount, char **tokens)
@@ -502,12 +513,15 @@ static void	listPlans()
 	Object	elt;
 		OBJ_POINTER(Dtn2Plan, plan);
 
+	CHKVOID(sdr_begin_xn(sdr));
 	for (elt = sdr_list_first(sdr, (getDtnConstants())->plans); elt;
 			elt = sdr_list_next(sdr, elt))
 	{
 		GET_OBJ_POINTER(sdr, Dtn2Plan, plan, sdr_list_data(sdr, elt));
 		printPlan(plan);
 	}
+
+	sdr_exit_xn(sdr);
 }
 
 static void	listRules(Dtn2Plan *plan)
@@ -526,6 +540,7 @@ static void	listRules(Dtn2Plan *plan)
 
 static void	executeList(int tokenCount, char **tokens)
 {
+	Sdr	sdr = getIonsdr();
 	Object	planAddr;
 		OBJ_POINTER(Dtn2Plan, plan);
 	Object	elt;
@@ -550,15 +565,20 @@ static void	executeList(int tokenCount, char **tokens)
 			return;
 		}
 
+		CHKVOID(sdr_begin_xn(sdr));
 		dtn2_findPlan(tokens[2], &planAddr, &elt);
 		if (elt == 0)
 		{
 			printText("Unknown plan.");
-			return;
+		}
+		else
+		{
+			GET_OBJ_POINTER(sdr, Dtn2Plan, plan, planAddr);
+			printPlan(plan);
+			listRules(plan);
 		}
 
-		GET_OBJ_POINTER(getIonsdr(), Dtn2Plan, plan, planAddr);
-		listRules(plan);
+		sdr_exit_xn(sdr);
 		return;
 	}
 
@@ -598,6 +618,7 @@ static int	processLine(char *line, int lineLength)
 	char	*cursor;
 	int	i;
 	char	*tokens[9];
+	char	buffer[80];
 
 	tokenCount = 0;
 	for (cursor = line, i = 0; i < 9; i++)
@@ -646,6 +667,12 @@ static int	processLine(char *line, int lineLength)
 			printUsage();
 			return 0;
 
+		case 'v':
+			isprintf(buffer, sizeof buffer, "%s",
+					IONVERSIONNUMBER);
+			printText(buffer);
+			return 0;
+
 		case 'a':
 			executeAdd(tokenCount, tokens);
 			return 0;
