@@ -36,12 +36,31 @@ static IpnDB	*_ipnConstants()
 {
 	static IpnDB	buf;
 	static IpnDB	*db = NULL;
+	Sdr		sdr;
+	Object		dbObject;
 
 	if (db == NULL)
 	{
-		sdr_read(getIonsdr(), (char *) &buf, _ipndbObject(NULL),
-				sizeof(IpnDB));
-		db = &buf;
+		sdr = getIonsdr();
+		CHKNULL(sdr);
+		dbObject = _ipndbObject(NULL);
+		if (dbObject)
+		{
+			if (sdr_heap_is_halted(sdr))
+			{
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(IpnDB));
+			}
+			else
+			{
+				CHKNULL(sdr_begin_xn(sdr));
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(IpnDB));
+				sdr_exit_xn(sdr);
+			}
+
+			db = &buf;
+		}
 	}
 
 	return db;
@@ -75,7 +94,7 @@ static int	lookupIpnEid(char *uriBuffer, char *neighborClId)
 			 *	neighbor's EID.				*/
 
 			isprintf(uriBuffer, SDRSTRING_BUFSZ,
-					"ipn:%lu.0", plan->nodeNbr);
+				"ipn:" UVAST_FIELDSPEC ".0", plan->nodeNbr);
 			return 1;
 		}
 	}
@@ -91,7 +110,7 @@ int	ipnInit()
 
 	/*	Recover the IPN database, creating it if necessary.	*/
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	oK(senderEidLookupFunctions(lookupIpnEid));
 	ipndbObject = sdr_find(sdr, IPN_DBNAME, NULL);
 	switch (ipndbObject)
@@ -142,7 +161,7 @@ IpnDB	*getIpnConstants()
 	return _ipnConstants();
 }
 
-static Object	locatePlan(unsigned long nodeNbr, Object *nextPlan)
+static Object	locatePlan(uvast nodeNbr, Object *nextPlan)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -175,7 +194,7 @@ static Object	locatePlan(unsigned long nodeNbr, Object *nextPlan)
 	return 0;
 }
 
-void	ipn_findPlan(unsigned long nodeNbr, Object *planAddr, Object *eltp)
+void	ipn_findPlan(uvast nodeNbr, Object *planAddr, Object *eltp)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -212,7 +231,7 @@ static void	createXmitDirective(FwdDirective *directive,
 	}
 }
 
-int	ipn_addPlan(unsigned long nodeNbr, DuctExpression *defaultDuct)
+int	ipn_addPlan(uvast nodeNbr, DuctExpression *defaultDuct)
 {
 	Sdr	sdr = getIonsdr();
 	Object	nextPlan;
@@ -220,7 +239,7 @@ int	ipn_addPlan(unsigned long nodeNbr, DuctExpression *defaultDuct)
 	Object	planObj;
 
 	CHKERR(nodeNbr && defaultDuct);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (locatePlan(nodeNbr, &nextPlan) != 0)
 	{
 		sdr_exit_xn(sdr);
@@ -266,7 +285,7 @@ static void	destroyXmitDirective(FwdDirective *directive)
 	}
 }
 
-int	ipn_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct)
+int	ipn_updatePlan(uvast nodeNbr, DuctExpression *defaultDuct)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -274,7 +293,7 @@ int	ipn_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct)
 	IpnPlan	plan;
 
 	CHKERR(nodeNbr && defaultDuct);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -299,7 +318,7 @@ int	ipn_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct)
 	return 1;
 }
 
-int	ipn_removePlan(unsigned long nodeNbr)
+int	ipn_removePlan(uvast nodeNbr)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -307,7 +326,7 @@ int	ipn_removePlan(unsigned long nodeNbr)
 		OBJ_POINTER(IpnPlan, plan);
 
 	CHKERR(nodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -341,8 +360,8 @@ int	ipn_removePlan(unsigned long nodeNbr)
 	return 1;
 }
 
-static Object	locateRule(Object rules, unsigned long srcServiceNbr,
-			unsigned long srcNodeNbr, Object *nextRule)
+static Object	locateRule(Object rules, unsigned int srcServiceNbr,
+			uvast srcNodeNbr, Object *nextRule)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -391,13 +410,13 @@ static Object	locateRule(Object rules, unsigned long srcServiceNbr,
 	return 0;
 }
 
-void	ipn_findPlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, IpnPlan *plan, Object *ruleAddr, Object *eltp)
+void	ipn_findPlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr,
+		IpnPlan *plan, Object *ruleAddr, Object *eltp)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnPlan, planPtr);
@@ -437,13 +456,13 @@ void	ipn_findPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	*eltp = elt;
 }
 
-int	ipn_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, DuctExpression *directive)
+int	ipn_addPlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr,
+		DuctExpression *directive)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnPlan, plan);
@@ -452,7 +471,7 @@ int	ipn_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	Object		addr;
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -500,13 +519,13 @@ int	ipn_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	return 1;
 }
 
-int	ipn_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, DuctExpression *directive)
+int	ipn_updatePlanRule(uvast nodeNbr, int argServiceNbr,
+		vast argNodeNbr, DuctExpression *directive)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnPlan, plan);
@@ -514,7 +533,7 @@ int	ipn_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
 	IpnRule		ruleBuf;
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -549,13 +568,12 @@ int	ipn_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
 	return 1;
 }
 
-int	ipn_removePlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr)
+int	ipn_removePlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnPlan, plan);
@@ -563,7 +581,7 @@ int	ipn_removePlanRule(unsigned long nodeNbr, long argServiceNbr,
 			OBJ_POINTER(IpnRule, rule);
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -598,15 +616,15 @@ int	ipn_removePlanRule(unsigned long nodeNbr, long argServiceNbr,
 	return 1;
 }
 
-static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
-			unsigned long sourceNodeNbr, FwdDirective *dirbuf)
+static int	lookupRule(Object rules, unsigned int sourceServiceNbr,
+			uvast sourceNodeNbr, FwdDirective *dirbuf)
 {
 	Sdr	sdr = getIonsdr();
 	Object	addr;
 	Object	elt;
 		OBJ_POINTER(IpnRule, rule);
 
-	/*	Universal wild-card match (IPN_ALL_OTHERS), if any,
+	/*	Universal wild-card match (IPN_ALL_OTHER_xxx), if any,
 	 *	is at the end of the list, so there's no way to
 	 *	terminate the search early.				*/
 
@@ -621,7 +639,7 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 		}
 
 		if (rule->srcServiceNbr > sourceServiceNbr
-			&& rule->srcServiceNbr != IPN_ALL_OTHERS)
+			&& rule->srcServiceNbr != IPN_ALL_OTHER_SERVICES)
 		{
 			continue;
 		}
@@ -634,7 +652,7 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 		}
 
 		if (rule->srcNodeNbr > sourceNodeNbr
-			&& rule->srcNodeNbr != IPN_ALL_OTHERS)
+			&& rule->srcNodeNbr != IPN_ALL_OTHER_NODES)
 		{
 			continue;
 		}
@@ -652,9 +670,8 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 	return 0;
 }
 
-int	ipn_lookupPlanDirective(unsigned long nodeNbr,
-		unsigned long sourceServiceNbr, unsigned long sourceNodeNbr,
-		FwdDirective *dirbuf)
+int	ipn_lookupPlanDirective(uvast nodeNbr, unsigned int sourceServiceNbr,
+		uvast sourceNodeNbr, FwdDirective *dirbuf)
 {
 	Sdr	sdr = getIonsdr();
 	Object	addr;
@@ -689,8 +706,8 @@ int	ipn_lookupPlanDirective(unsigned long nodeNbr,
 	return 1;
 }
 
-static Object	locateGroup(unsigned long firstNodeNbr,
-			unsigned long lastNodeNbr, Object *nextGroup)
+static Object	locateGroup(uvast firstNodeNbr, uvast lastNodeNbr,
+			Object *nextGroup)
 {
 	Sdr	sdr = getIonsdr();
 	int	targetSize;
@@ -740,8 +757,8 @@ static Object	locateGroup(unsigned long firstNodeNbr,
 	return 0;
 }
 
-void	ipn_findGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		Object *groupAddr, Object *eltp)
+void	ipn_findGroup(uvast firstNodeNbr, uvast lastNodeNbr, Object *groupAddr,
+		Object *eltp)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -763,8 +780,7 @@ void	ipn_findGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	*eltp = elt;
 }
 
-int	ipn_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		char *viaEid)
+int	ipn_addGroup(uvast firstNodeNbr, uvast lastNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
 	Object		nextGroup;
@@ -774,7 +790,7 @@ int	ipn_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	CHKERR(firstNodeNbr && lastNodeNbr && viaEid);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
 	CHKERR(strlen(viaEid) <= MAX_SDRSTRING);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (locateGroup(firstNodeNbr, lastNodeNbr, &nextGroup) != 0)
 	{
 		sdr_exit_xn(sdr);
@@ -815,8 +831,7 @@ int	ipn_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	ipn_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		char *viaEid)
+int	ipn_updateGroup(uvast firstNodeNbr, uvast lastNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
 	Object		elt;
@@ -826,7 +841,7 @@ int	ipn_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	CHKERR(firstNodeNbr && lastNodeNbr && viaEid);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
 	CHKERR(strlen(viaEid) <= MAX_SDRSTRING);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -851,7 +866,7 @@ int	ipn_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	ipn_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
+int	ipn_removeGroup(uvast firstNodeNbr, uvast lastNodeNbr)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -860,7 +875,7 @@ int	ipn_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
 
 	CHKERR(firstNodeNbr && lastNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -893,14 +908,14 @@ int	ipn_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
 	return 1;
 }
 
-void	ipn_findGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		long argServiceNbr, long argNodeNbr, IpnGroup *group,
+void	ipn_findGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, IpnGroup *group,
 		Object *ruleAddr, Object *eltp)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, groupPtr);
@@ -941,13 +956,13 @@ void	ipn_findGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	*eltp = elt;
 }
 
-int	ipn_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		long argServiceNbr, long argNodeNbr, char *viaEid)
+int	ipn_addGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -957,7 +972,7 @@ int	ipn_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1004,14 +1019,13 @@ int	ipn_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	ipn_updateGroupRule(unsigned long firstNodeNbr,
-		unsigned long lastNodeNbr, long argServiceNbr, long argNodeNbr,
-		char *viaEid)
+int	ipn_updateGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -1020,7 +1034,7 @@ int	ipn_updateGroupRule(unsigned long firstNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1054,13 +1068,13 @@ int	ipn_updateGroupRule(unsigned long firstNodeNbr,
 	return 1;
 }
 
-int	ipn_removeGroupRule(unsigned long firstNodeNbr,
-		unsigned long lastNodeNbr, long argServiceNbr, long argNodeNbr)
+int	ipn_removeGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? IPN_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				IPN_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? IPN_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -1069,7 +1083,7 @@ int	ipn_removeGroupRule(unsigned long firstNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1103,9 +1117,8 @@ int	ipn_removeGroupRule(unsigned long firstNodeNbr,
 	return 1;
 }
 
-int	ipn_lookupGroupDirective(unsigned long nodeNbr,
-		unsigned long sourceServiceNbr, unsigned long sourceNodeNbr,
-		FwdDirective *dirbuf)
+int	ipn_lookupGroupDirective(uvast nodeNbr, unsigned int sourceServiceNbr,
+		uvast sourceNodeNbr, FwdDirective *dirbuf)
 {
 	Sdr		sdr = getIonsdr();
 	Object		elt;
