@@ -78,6 +78,16 @@ extern "C" {
 #define BP_MAX_BLOCK_SIZE		(2000)
 #endif
 
+/*	We hitchhike on the ZCO heap space management system to 
+ *	manage the space occupied by Bundle objects.  In effect,
+ *	the Bundle overhead objects compete with ZCOs for available
+ *	SDR heap space.  We don't want this practice to become
+ *	widespread, which is why these functions are declared
+ *	privately here rather than publicly in the zco.h header.	*/
+
+extern void	zco_increase_heap_occupancy(Sdr sdr, vast delta);
+extern void	zco_reduce_heap_occupancy(Sdr sdr, vast delta);
+
 /*	A BP "node" is a set of cooperating state machines that
  *	together constitute a single functional point of presence,
  *	residing in a single SDR database, in a DTN-based network.
@@ -105,9 +115,14 @@ extern "C" {
 
 typedef struct
 {
-	long		nominalRate;	/*	In bytes per second.	*/
-	long		capacity;	/*	Bytes, current second.	*/
+	uvast		nbr;
+} NodeId;
+
+typedef struct
+{
 	sm_SemId	semaphore;
+	int		nominalRate;	/*	In bytes per second.	*/
+	vast		capacity;	/*	Bytes, current second.	*/
 } Throttle;
 
 typedef struct
@@ -124,8 +139,8 @@ typedef struct
 
 typedef struct
 {
-	unsigned long	nodeNbr;
-	unsigned long	serviceNbr;
+	uvast		nodeNbr;	/*	For "imc", group nbr.	*/
+	unsigned int	serviceNbr;
 } CbheEid;
 
 typedef struct
@@ -133,12 +148,13 @@ typedef struct
 	DtnEid		d;
 	CbheEid		c;
 	char		cbhe;		/*	Boolean.		*/
+	char		unicast;	/*	Boolean.		*/
 } EndpointId;
 
 typedef struct
 {
-	unsigned long	nodeNbr;
-	unsigned long	serviceNbr;
+	uvast		nodeNbr;
+	unsigned int	serviceNbr;
 	char		*schemeName;
 	int		schemeNameLength;
 	char		*colon;
@@ -167,19 +183,18 @@ typedef struct
 {
 	EndpointId	source;		/*	Original sender.	*/
 	BpTimestamp	creationTime;
-	unsigned long	fragmentOffset;
+	unsigned int	fragmentOffset;
 } BundleId;
 
 typedef struct
 {
-	unsigned long	length;		/*	initial length of ZCO	*/
+	unsigned int	length;		/*	initial length of ZCO	*/
 	Object		content;	/*	a ZCO reference in SDR	*/
 } Payload;
 
 /*	Administrative record types	*/
 #define	BP_STATUS_REPORT	(1)
 #define	BP_CUSTODY_SIGNAL	(2)
-#define	BP_AGGREGATE_CUSTODY_SIGNAL	(4)
 
 /*	Administrative record flags	*/
 #define BP_BDL_IS_A_FRAGMENT	(1)	/*	00000001		*/
@@ -198,15 +213,15 @@ typedef enum
 
 typedef struct
 {
-	unsigned long	seconds;
-	unsigned long	nanosec;
+	unsigned int	seconds;
+	unsigned int	nanosec;
 } DtnTime;
 
 typedef struct
 {
 	BpTimestamp	creationTime;	/*	From bundle's ID.	*/
-	unsigned long	fragmentOffset;	/*	From bundle's ID.	*/
-	unsigned long	fragmentLength;
+	unsigned int	fragmentOffset;	/*	From bundle's ID.	*/
+	unsigned int	fragmentLength;
 	char		*sourceEid;	/*	From bundle's ID.	*/
 	unsigned char	isFragment;	/*	Boolean.		*/
 	unsigned char	flags;
@@ -231,8 +246,8 @@ typedef enum
 typedef struct
 {
 	BpTimestamp	creationTime;	/*	From bundle's ID.	*/
-	unsigned long	fragmentOffset;	/*	From bundle's ID.	*/
-	unsigned long	fragmentLength;
+	unsigned int	fragmentOffset;	/*	From bundle's ID.	*/
+	unsigned int	fragmentLength;
 	char		*sourceEid;	/*	From bundle's ID.	*/
 	unsigned char	isFragment;	/*	Boolean.		*/
 	unsigned char	succeeded;	/*	Boolean.		*/
@@ -248,7 +263,7 @@ typedef struct
 {
 	int		authentic;	/*	Boolean.		*/
 	BpString	senderEid;
-	unsigned long	senderNodeNbr;	/*	If CBHE.		*/
+	uvast		senderNodeNbr;	/*	If CBHE.		*/
 } ClDossier;
 
 /*	Bundle processing flags						*/
@@ -278,25 +293,31 @@ typedef struct
 
 	/*	Stuff in Primary block.					*/
 
-	unsigned long	bundleProcFlags;	/*	Incl. CoS, SRR.	*/
+	unsigned int	bundleProcFlags;/*	Incl. CoS, SRR.		*/
+	unsigned int	timeToLive;	/*	In seconds.		*/
 	EndpointId	destination;	/*	...of bundle's ADU	*/
 		/*	source of bundle's ADU is in the id field.	*/
 	EndpointId	reportTo;
 	EndpointId	custodian;
 		/*	creation time is in the id field.		*/
-	unsigned long	expirationTime;	/*	Time tag in seconds.	*/
-	unsigned long	dictionaryLength;
+	unsigned int	expirationTime;	/*	Time tag in seconds.	*/
+	unsigned int	dictionaryLength;
 	Object		dictionary;
 		/*	fragment offset is in the id field.		*/
-	unsigned long	totalAduLength;
+	unsigned int	totalAduLength;
 
 	/*	Stuff in Extended COS extension block.			*/
 
 	BpExtendedCOS	extendedCOS;
 
+	/*	Stuff in (or for) the Bundle Age extension block.	*/
+
+	unsigned int	age;		/*	In microseconds.	*/
+	struct timeval	arrivalTime;
+
 	/*	Stuff in Payload block.					*/
 
-	unsigned long	payloadBlockProcFlags;
+	unsigned int	payloadBlockProcFlags;
 	Payload		payload;
 
 	/*	Stuff in extension blocks, preceding and following the
@@ -319,7 +340,6 @@ typedef struct
 	char		corrupt;	/*	Boolean.		*/
 	char		anonymous;	/*	Boolean.		*/
 	int		dbOverhead;	/*	SDR bytes occupied.	*/
-	int		dbTotal;	/*	Overhead + payload len.	*/
 	BpStatusRpt	statusRpt;	/*	For response per CoS.	*/
 	BpCtSignal	ctSignal;	/*	For acknowledgement.	*/
 	ClDossier	clDossier;	/*	Processing hints.	*/
@@ -368,7 +388,7 @@ typedef struct
 	 *	the application, and all fragments (including the
 	 *	final one) are destroyed.				*/
 
-	unsigned long	totalAduLength;
+	unsigned int	totalAduLength;
 } IncompleteBundle;
 
 /*	*	*	Endpoint structures	*	*	*	*/
@@ -427,7 +447,9 @@ typedef struct
 typedef struct
 {
 	char		name[MAX_SCHEME_NAME_LEN + 1];
+	int		nameLength;
 	int		cbhe;		/*	Boolean.		*/
+	int		unicast;	/*	Boolean;		*/
 	Object		fwdCmd; 	/*	For starting forwarder.	*/
 	Object		admAppCmd; 	/*	For starting admin app.	*/
 	Object		forwardQueue;	/*	SDR list of Bundles	*/
@@ -437,11 +459,16 @@ typedef struct
 typedef struct
 {
 	Object		schemeElt;	/*	Reference to scheme.	*/
+
+	/*	Copied from Scheme.	*/
+
 	char		name[MAX_SCHEME_NAME_LEN + 1];
-	char		custodianEidString[MAX_EID_LEN];
-	int		custodianSchemeNameLength;
-	int		custodianNssLength;
-	int		cbhe;		/*	Copied from Scheme.	*/
+	int		nameLength;
+	int		cbhe;
+	int		unicast;
+
+	char		adminEid[MAX_EID_LEN];
+	int		adminNSSLength;
 	int		fwdPid;		/*	For stopping forwarder.	*/
 	int		admAppPid;	/*	For stopping admin app.	*/
 	sm_SemId	semaphore;	/*	For dispatch notices.	*/
@@ -501,7 +528,7 @@ typedef struct
 	Object		urgentQueue;	/*	SDR list of Bundles	*/
 	Scalar		urgentBacklog;	/*	Urgent bytes enqueued.	*/
 	OrdinalState	ordinals[256];	/*	Orders urgent queue.	*/
-	unsigned long	maxPayloadLen;	/*	0 = no limit.		*/
+	unsigned int	maxPayloadLen;	/*	0 = no limit.		*/
 	int		blocked;	/*	Boolean			*/
 	Object		protocol;	/*	back-reference		*/
 	Object		stats;		/*	OutductStats address.	*/
@@ -537,7 +564,7 @@ typedef struct
 	char		name[MAX_CL_PROTOCOL_NAME_LEN + 1];
 	int		payloadBytesPerFrame;
 	int		overheadPerFrame;
-	long		nominalRate;	/*	Bytes per second.	*/
+	int		nominalRate;	/*	Bytes per second.	*/
 	Object		inducts;	/*	SDR list of Inducts	*/
 	Object		outducts;	/*	SDR list of Outducts	*/
 } ClProtocol;
@@ -576,8 +603,9 @@ typedef struct
 	Object		inboundBundles;	/*	SDR list of ZCOs	*/
 	Object		limboQueue;	/*	SDR list of Bundles	*/
 	Object		clockCmd; 	/*	For starting clock.	*/
-	BpString	custodianEidString;
 	int		maxAcqInHeap;
+	unsigned int	bundleCounter;	/*	For non-synced clock.	*/
+	int		watching;	/*	Activity watch switch.	*/
 	time_t		resetTime;	/*	Stats reset time.	*/
 	Object		sourceStats;	/*	BpCosStats address.	*/
 	Object		recvStats;	/*	BpCosStats address.	*/
@@ -593,7 +621,7 @@ typedef struct
 
 typedef struct
 {
-	unsigned long	ctExpiredTimeout;
+	unsigned int	ctExpiredTimeout;
 } BpctDB;
 
 #define BP_STATUS_RECEIVE	0
@@ -670,19 +698,14 @@ typedef struct
 	Object		ctStats;	/*	BpCtStats address.	*/
 	Object		dbStats;	/*	BpDbStats address.	*/
 	int		updateStats;	/*	Boolean.		*/
-	unsigned long	creationTimeSec;
+	unsigned int	creationTimeSec;
 	int		bundleCounter;
 	int		clockPid;	/*	For stopping clock.	*/
 	int		watching;	/*	Activity watch switch.	*/
 
-	/*	For congestion control.					*/
-
-	Throttle	productionThrottle;
-
 	/*	For finding structures in database.			*/
 
 	PsmAddress	schemes;	/*	SM list: VScheme.	*/
-	PsmAddress	cbheScheme;	/*	A single VScheme.	*/
 	PsmAddress	inducts;	/*	SM list: VInduct.	*/
 	PsmAddress	outducts;	/*	SM list: VOutduct.	*/
 	PsmAddress	timeline;	/*	SM RB tree: list xref.	*/
@@ -714,6 +737,7 @@ typedef struct
 	AcqDecision	decision;
 	int		lastBlockParsed;
 	int		malformed;
+	int		congestive;	/*	Not enough ZCO space.	*/
 	int		mustAbort;	/*	Unreadable block(s).	*/
 
 	/*	Per-acquisition state variables.			*/
@@ -905,7 +929,7 @@ extern int		bpDequeue(	VOutduct *vduct,
 					Object *outboundZco,
 					BpExtendedCOS *extendedCOS,
 					char *destDuctName,
-					unsigned long maxPayloadLength,
+					unsigned int maxPayloadLength,
 					int timeoutInterval);
 			/*	This function is invoked by a
 			 *	convergence-layer output adapter (an
@@ -1209,48 +1233,14 @@ extern int		bpDestroyBundle(Object bundleToDestroy,
 			 *	retained because not all constraints
 			 *	have been removed, -1 on any error.	*/
 
-extern int		bpConstructStatusRpt(BpStatusRpt *rpt,
-					Object *payloadZco);
-			/*	Catenates (serializes) rpt as the
-			 *	source data of a new ZCO and passes
-			 *	back the address of that new ZCO.
-			 *
-			 *	Returns 0 on success, -1 on any error.	*/
-
-extern void		bpEraseStatusRpt(BpStatusRpt *rpt);
-			/*	Frees any dynamic memory used in
-			 *	expressing this status report.		*/
-
-extern int		bpConstructCtSignal(BpCtSignal *signal,
-					Object *payloadZco);
-			/*	Catenates (serializes) signal as the
-			 *	source data of a new ZCO and passes
-			 *	back the address of that new ZCO.
-			 *
-			 *	Returns 0 on success, -1 on any error.	*/
-
-extern void		bpEraseCtSignal(BpCtSignal *signal);
-			/*	Frees any dynamic memory used in
-			 *	expressing this custody transfer signal.*/
-
-extern int		bpParseAdminRecord(int *adminRecordType,
-					BpStatusRpt *rpt,
-					BpCtSignal *signal,
-					void **acsptr,
-					Object payload);
-			/*	Populates the appropriate structure
-			 *	from payload content and notes the
-			 *	corresponding admin record type (0
-			 *	if payload is not an admin record).
-			 *
-			 *	Returns 1 on success, 0 on parsing
-			 *	failure, -1 on any other error.		*/
-
 extern int		bpInit();
+extern void		bpDropVdb();
+extern void		bpRaiseVdb();
 extern int		bpSetCTCountdownTimer(time_t newTimeout);
 extern int		bpStart();
 extern void		bpStop();
 extern int		bpAttach();
+extern void		bpDetach();
 
 extern Object		getBpDbObject();
 extern BpDB		*getBpConstants();
@@ -1300,7 +1290,7 @@ extern int		removeEndpoint(char *endpointName);
 
 extern void		fetchProtocol(char *name, ClProtocol *clp, Object *elt);
 extern int		addProtocol(char *name, int payloadBytesPerFrame,
-				int overheadPerFrame, long nominalRate);
+				int overheadPerFrame, int nominalRate);
 extern int		removeProtocol(char *name);
 extern int		bpStartProtocol(char *name);
 extern void		bpStopProtocol(char *name);
@@ -1318,9 +1308,9 @@ extern void		bpStopInduct(char *protocolName, char *ductName);
 extern void		findOutduct(char *protocolName, char *name,
 				VOutduct **vduct, PsmAddress *elt);
 extern int		addOutduct(char *protocolName, char *name,
-				char *cloCmd, unsigned long maxPayloadLength);
+				char *cloCmd, unsigned int maxPayloadLength);
 extern int		updateOutduct(char *protocolName, char *name,
-				char *cloCmd, unsigned long maxPayloadLength);
+				char *cloCmd, unsigned int maxPayloadLength);
 extern int		removeOutduct(char *protocolName, char *name);
 extern int		bpStartOutduct(char *protocolName, char *ductName);
 extern void		bpStopOutduct(char *protocolName, char *ductName);
@@ -1331,8 +1321,8 @@ extern Object		insertBpTimelineEvent(BpEvent *newEvent);
 extern void		destroyBpTimelineEvent(Object timelineElt);
 
 extern int		findBundle(char *sourceEid, BpTimestamp *creationTime,
-				unsigned long fragmentOffset,
-				unsigned long fragmentLength,
+				unsigned int fragmentOffset,
+				unsigned int fragmentLength,
 				Object *bundleAddr);
 extern int		retrieveInTransitBundle(Object bundleZco, Object *obj);
 
@@ -1358,12 +1348,11 @@ typedef struct bpsap_st
 	sm_SemId	recvSemaphore;
 } Sap;
 
-extern int		handleAbstractCtSignal(BpCtSignal *, char *);
 extern int		_handleAdminBundles(char *adminEid,
 				StatusRptCB handleStatusRpt,
 				CtSignalCB handleCtSignal);
-
-extern int eidIsLocal(EndpointId eid, char* dictionary);
+extern int		applyCtSignal(BpCtSignal *, char *);
+extern int		eidIsLocal(EndpointId eid, char* dictionary);
 
 #ifdef __cplusplus
 }
