@@ -36,7 +36,6 @@
 
 #define GMSG_BUFLEN     256
 #if BAB_DEBUGGING == 1
-extern char             gMsg[];         /*      Debug message buffer.   */
 
 /**
  * \def BAB_DEBUG
@@ -110,10 +109,10 @@ extern char             gMsg[];         /*      Debug message buffer.   */
 typedef struct
 {
 	CollabBlockHdr hdr;
-	unsigned long correlator;
-	unsigned long cipher;
+	unsigned int correlator;
+	unsigned int cipher;
 	char cipherKeyName[BSP_KEY_NAME_LEN]; /** Cipherkey name used by this block.*/
-	unsigned long rxFlags;        /** RX-side processing flags for this block. */
+	unsigned int rxFlags;        /** RX-side processing flags for this block. */
 	int hmacLen;
 	char expectedResult[BAB_HMAC_SHA1_RESULT_LEN];
 } BspBabCollaborationBlock;
@@ -463,10 +462,10 @@ void bsp_babRelease(ExtensionBlock *blk);
 
 
 unsigned char *bsp_babGetSecResult(Object dataObj,
-                                   unsigned long dataLen,
+                                   unsigned int dataLen,
                                    char *cipherKeyName,
-				   unsigned long keyLen,
-                                   unsigned long *hashLen);
+				   unsigned int keyLen,
+                                   unsigned int *hashLen);
 
 
 
