@@ -12,21 +12,33 @@
 #include "bpP.h"
 #include "bpnm.h"
 
-void    bpnm_resources(unsigned long * heapOccupancyLimit,
-		unsigned long * heapMaxOccupancyForecast,
-		unsigned long * heapOccupancy)
+void    bpnm_resources(double * occupancyCeiling,
+		double * maxForecastOccupancy,
+		double * currentOccupancy,
+		double * maxHeapOccupancy,
+		double * currentHeapOccupancy,
+		double * maxFileOccupancy,
+		double * currentFileOccupancy)
 {
     Sdr     sdr = getIonsdr();
     IonDB   db;
 
-    CHKVOID(heapOccupancyLimit);
-    CHKVOID(heapMaxOccupancyForecast);
-    CHKVOID(heapOccupancy);
-    sdr_begin_xn(sdr);
+    CHKVOID(occupancyCeiling);
+    CHKVOID(maxForecastOccupancy);
+    CHKVOID(currentOccupancy);
+    CHKVOID(maxHeapOccupancy);
+    CHKVOID(currentHeapOccupancy);
+    CHKVOID(maxFileOccupancy);
+    CHKVOID(currentFileOccupancy);
+    CHKVOID(sdr_begin_xn(sdr));
     sdr_read(sdr, (char *) &db, getIonDbObject(), sizeof(IonDB));
-    *heapOccupancyLimit = db.occupancyCeiling;
-    *heapMaxOccupancyForecast = db.maxForecastOccupancy;
-    *heapOccupancy = db.currentOccupancy;
+    *occupancyCeiling = db.occupancyCeiling;
+    *maxForecastOccupancy = db.maxForecastOccupancy;
+    *maxHeapOccupancy = zco_get_max_heap_occupancy(sdr);
+    *currentHeapOccupancy = zco_get_heap_occupancy(sdr);
+    *maxFileOccupancy = zco_get_max_file_occupancy(sdr);
+    *currentFileOccupancy = zco_get_file_occupancy(sdr);
+    *currentOccupancy = *currentHeapOccupancy + *currentFileOccupancy;
     sdr_exit_xn(sdr);
 }
 
@@ -170,7 +182,7 @@ static void getBpEndpoint(char * targetName, VEndpoint **vpoint, int  * success)
 void bpnm_endpoint_get (char * targetName, NmbpEndpoint * results, int * success)
 {
     Sdr             sdr = getIonsdr();
-    VEndpoint     * vpoint;
+    VEndpoint     * vpoint = NULL;
     Endpoint        endpoint;
     Object          elt;
     Bundle          bundle;
@@ -189,7 +201,7 @@ void bpnm_endpoint_get (char * targetName, NmbpEndpoint * results, int * success
         /* Copy extra byte to ensure that the NULL byte gets copied. */
         memcpy (results->eid, targetName, strlen (targetName)+1 );
     
-        sdr_begin_xn(sdr);
+        CHKVOID(sdr_begin_xn(sdr));
         sdr_read(sdr, (char *) & endpoint, sdr_list_data(sdr, vpoint->endpointElt), sizeof(Endpoint));
 	results->currentQueuedBundlesCount = sdr_list_length(sdr, endpoint.deliveryQueue);
 	results->currentQueuedBundlesBytes = 0;
@@ -262,7 +274,7 @@ void  bpnm_endpoint_reset (char * targetName, int * success)
                      
             if (strcmp (targetName, computedName) == 0)
             {
-                sdr_begin_xn(sdr);
+                CHKVOID(sdr_begin_xn(sdr));
                 sdr_stage(sdr, (char *) & stats, vpoint->stats, sizeof(EndpointStats));
 
                 stats.resetTime = getUTCTime();
@@ -274,7 +286,7 @@ void  bpnm_endpoint_reset (char * targetName, int * success)
                 }
 
                 sdr_write(sdr, vpoint->stats, (char *) & stats, sizeof(EndpointStats));
-                sdr_end_xn(sdr);
+                oK(sdr_end_xn(sdr));
           
                 * success = 1;
                 return;
@@ -404,7 +416,7 @@ static void getBpInductStats (char * targetName, VInduct ** vduct, int  * succes
 void bpnm_induct_get (char * targetName, NmbpInduct * results, int * success)
 {
     Sdr             sdr = getIonsdr();
-    VInduct       * vduct;
+    VInduct       * vduct = NULL;
     Induct          duct;
     InductStats     stats;
 
@@ -421,7 +433,7 @@ void bpnm_induct_get (char * targetName, NmbpInduct * results, int * success)
         /* Copy extra byte to ensure that the NULL byte gets copied. */
         memcpy (results->inductName, targetName, strlen (targetName)+1 );
     
-        sdr_begin_xn(sdr);
+        CHKVOID(sdr_begin_xn(sdr));
         sdr_read(sdr, (char *) & duct, sdr_list_data(sdr, vduct->inductElt), sizeof(Induct));
         sdr_read(sdr, (char *) & stats, duct.stats, sizeof(InductStats));
         sdr_exit_xn(sdr);
@@ -489,7 +501,7 @@ void bpnm_induct_reset (char * targetName, int * success)
 
                 if (strcmp (targetName, computedName) == 0)
                 {
-                    sdr_begin_xn(sdr);
+                    CHKVOID(sdr_begin_xn(sdr));
                     sdr_stage(sdr, (char *) & stats, vduct->stats, sizeof(InductStats));
 
                     stats.resetTime = getUTCTime();
@@ -501,7 +513,7 @@ void bpnm_induct_reset (char * targetName, int * success)
                     }
     
                     sdr_write(sdr, vduct->stats, (char *) & stats, sizeof(InductStats));
-                    sdr_end_xn(sdr);
+                    oK(sdr_end_xn(sdr));
               
                     * success = 1;
                     return;
@@ -630,7 +642,7 @@ static void getBpOutductStats (char * targetName, VOutduct ** vduct, int  * succ
 void bpnm_outduct_get (char * targetName, NmbpOutduct * results, int * success)
 {
     Sdr             sdr = getIonsdr();
-    VOutduct      * vduct;
+    VOutduct      * vduct = NULL;
     Outduct         duct;
     OutductStats    stats;
     
@@ -647,7 +659,7 @@ void bpnm_outduct_get (char * targetName, NmbpOutduct * results, int * success)
         /* Copy extra byte to ensure that the NULL byte gets copied. */
         memcpy (results->outductName, targetName, strlen (targetName)+1 );
 
-        sdr_begin_xn(sdr);
+        CHKVOID(sdr_begin_xn(sdr));
         sdr_read(sdr, (char *) & duct, sdr_list_data(sdr, vduct->outductElt), sizeof(Outduct));
         results->currentQueuedBundlesCount =
                 sdr_list_length(sdr, duct.urgentQueue)
@@ -718,7 +730,7 @@ void bpnm_outduct_reset (char * targetName, int * success)
 
                 if (strcmp (targetName, computedName) == 0)
                 {
-                    sdr_begin_xn(sdr);
+                    CHKVOID(sdr_begin_xn(sdr));
                     sdr_stage(sdr, (char *) & stats, vduct->stats, sizeof(OutductStats));
 
                     stats.resetTime = getUTCTime();
@@ -730,7 +742,7 @@ void bpnm_outduct_reset (char * targetName, int * success)
                     }
     
                     sdr_write(sdr, vduct->stats, (char *) & stats, sizeof(OutductStats));
-                    sdr_end_xn(sdr);
+                    oK(sdr_end_xn(sdr));
               
                     * success = 1;
                     return;
@@ -754,7 +766,7 @@ void    bpnm_limbo_get(NmbpOutduct * results)
 
     CHKVOID(results);
     istrcpy(results->outductName, "limbo", BPNM_OUTDUCT_NAME_LEN);
-    sdr_begin_xn(sdr);
+    CHKVOID(sdr_begin_xn(sdr));
     sdr_read(sdr, (char *) &bpdb, bpDbObject, sizeof(BpDB));
     results->currentQueuedBundlesCount = sdr_list_length(sdr, bpdb.limboQueue);
     results->currentQueuedBundlesBytes = 0;
@@ -801,7 +813,7 @@ void    bpnm_disposition_get(NmbpDisposition * results)
     BpDbStats       dbStats;
 
     CHKVOID(results);
-    sdr_begin_xn(sdr);
+    CHKVOID(sdr_begin_xn(sdr));
     sdr_read(sdr, (char *) &bpdb, bpDbObject, sizeof(BpDB));
     results->lastResetTime = bpdb.resetTime;
 
@@ -1130,7 +1142,7 @@ void    bpnm_disposition_reset()
     BpDB            db;
 
     dbobj = getBpDbObject();
-    sdr_begin_xn(sdr);
+    CHKVOID(sdr_begin_xn(sdr));
     sdr_stage(sdr, (char *) &db, dbobj, sizeof(BpDB));
     db.resetTime = getUTCTime();
     resetSourceStats(&db);
@@ -1141,6 +1153,6 @@ void    bpnm_disposition_reset()
     resetCtStats(&db);
     resetDbStats(&db);
     sdr_write(sdr, dbobj, (char *) &db, sizeof(BpDB));
-    sdr_end_xn(sdr);
+    oK(sdr_end_xn(sdr));
     
 }   /* end of bpnm_disposition_reset */
