@@ -10,6 +10,7 @@
 /*									*/
 
 #include "bpP.h"
+#include "crypto.h"
 
 static int	_echo(int *newValue)
 {
@@ -62,6 +63,7 @@ static void	printUsage()
 	PUTS("\tq\tQuit");
 	PUTS("\th\tHelp");
 	PUTS("\t?\tHelp");
+	PUTS("\tv\tPrint version of ION and crypto suite.");
 	PUTS("\t1\tInitialize");
 	PUTS("\t   1");
 	PUTS("\ta\tAdd");
@@ -210,8 +212,8 @@ static void	executeAdd(int tokenCount, char **tokens)
 {
 	char		*script;
 	BpRecvRule	rule;
-	long		nominalRate = 0;
-	unsigned long	maxPayloadLength;
+	int		nominalRate = 0;
+	unsigned int	maxPayloadLength;
 
 	if (tokenCount < 2)
 	{
@@ -319,7 +321,7 @@ static void	executeChange(int tokenCount, char **tokens)
 {
 	char		*script;
 	BpRecvRule	rule;
-	unsigned long	maxPayloadLen;
+	unsigned int	maxPayloadLen;
 
 	if (tokenCount < 2)
 	{
@@ -514,6 +516,7 @@ admpid: %d cmd %.256s", scheme->name, vscheme->fwdPid, fwdCmd,
 
 static void	infoScheme(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	VScheme		*vscheme;
 	PsmAddress	elt;
 
@@ -523,19 +526,23 @@ static void	infoScheme(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	findScheme(tokens[2], &vscheme, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown scheme.");
-		return;
+	}
+	else
+	{
+		printScheme(vscheme);
 	}
 
-	printScheme(vscheme);
+	sdr_exit_xn(sdr);
 }
 
 static void	printEndpoint(VEndpoint *vpoint)
 {
-	Sdr	sdr = getIonsdr();
+	Sdr		sdr = getIonsdr();
 		OBJ_POINTER(Endpoint, endpoint);
 		OBJ_POINTER(Scheme, scheme);
 	char	buffer[512];
@@ -575,6 +582,7 @@ static void	printEndpoint(VEndpoint *vpoint)
 
 static void	infoEndpoint(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	VEndpoint	*vpoint;
 	PsmAddress	elt;
 
@@ -584,14 +592,18 @@ static void	infoEndpoint(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	findEndpoint(tokens[2], tokens[3], NULL, &vpoint, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown endpoint.");
-		return;
+	}
+	else
+	{
+		printEndpoint(vpoint);
 	}
 
-	printEndpoint(vpoint);
+	sdr_exit_xn(sdr);
 }
 
 static void	printProtocol(ClProtocol *protocol)
@@ -601,6 +613,7 @@ static void	printProtocol(ClProtocol *protocol)
 
 static void	infoProtocol(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	Object		elt;
 	ClProtocol	clpbuf;
 
@@ -610,19 +623,23 @@ static void	infoProtocol(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	fetchProtocol(tokens[2], &clpbuf, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown protocol.");
-		return;
+	}
+	else
+	{
+		printProtocol(&clpbuf);
 	}
 
-	printProtocol(&clpbuf);
+	sdr_exit_xn(sdr);
 }
 
 static void	printInduct(VInduct *vduct)
 {
-	Sdr	sdr = getIonsdr();
+	Sdr		sdr = getIonsdr();
 		OBJ_POINTER(Induct, duct);
 		OBJ_POINTER(ClProtocol, clp);
 	char	cliCmdBuffer[SDRSTRING_BUFSZ];
@@ -648,6 +665,7 @@ static void	printInduct(VInduct *vduct)
 
 static void	infoInduct(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	VInduct		*vduct;
 	PsmAddress	elt;
 
@@ -657,19 +675,23 @@ static void	infoInduct(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	findInduct(tokens[2], tokens[3], &vduct, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown induct.");
-		return;
+	}
+	else
+	{
+		printInduct(vduct);
 	}
 
-	printInduct(vduct);
+	sdr_exit_xn(sdr);
 }
 
 static void	printOutduct(VOutduct *vduct)
 {
-	Sdr	sdr = getIonsdr();
+	Sdr		sdr = getIonsdr();
 		OBJ_POINTER(Outduct, duct);
 		OBJ_POINTER(ClProtocol, clp);
 	char	cloCmdBuffer[SDRSTRING_BUFSZ];
@@ -679,6 +701,7 @@ static void	printOutduct(VOutduct *vduct)
 	GET_OBJ_POINTER(sdr, Outduct, duct, sdr_list_data(sdr,
 			vduct->outductElt));
 	GET_OBJ_POINTER(sdr, ClProtocol, clp, duct->protocol);
+
 	if (duct->cloCmd == 0)
 	{
 		cloCmd = "?";
@@ -699,6 +722,7 @@ max: %lu", clp->name, duct->name, vduct->cloPid, cloCmd, duct->maxPayloadLen);
 
 static void	infoOutduct(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	VOutduct	*vduct;
 	PsmAddress	elt;
 
@@ -708,14 +732,18 @@ static void	infoOutduct(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	findOutduct(tokens[2], tokens[3], &vduct, &elt);
 	if (elt == 0)
 	{
 		printText("Unknown outduct.");
-		return;
+	}
+	else
+	{
+		printOutduct(vduct);
 	}
 
-	printOutduct(vduct);
+	sdr_exit_xn(sdr);
 }
 
 static void	executeInfo(int tokenCount, char **tokens)
@@ -761,6 +789,7 @@ static void	executeInfo(int tokenCount, char **tokens)
 
 static void	listSchemes(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
 	PsmAddress	elt;
 	VScheme		*vscheme;
@@ -771,12 +800,15 @@ static void	listSchemes(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	for (elt = sm_list_first(ionwm, (getBpVdb())->schemes); elt;
 			elt = sm_list_next(ionwm, elt))
 	{
 		vscheme = (VScheme *) psp(ionwm, sm_list_data(ionwm, elt));
 		printScheme(vscheme);
 	}
+
+	sdr_exit_xn(sdr);
 }
 
 static void	listEndpointsForScheme(VScheme *vscheme)
@@ -795,6 +827,7 @@ static void	listEndpointsForScheme(VScheme *vscheme)
 
 static void	listEndpoints(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
 	VScheme		*vscheme;
 	PsmAddress	elt;
@@ -802,6 +835,7 @@ static void	listEndpoints(int tokenCount, char **tokens)
 	switch (tokenCount)
 	{
 	case 2:
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sm_list_first(ionwm, (getBpVdb())->schemes); elt;
 				elt = sm_list_next(ionwm, elt))
 		{
@@ -810,17 +844,22 @@ static void	listEndpoints(int tokenCount, char **tokens)
 			listEndpointsForScheme(vscheme);
 		}
 
+		sdr_exit_xn(sdr);
 		break;
 
 	case 3:
+		CHKVOID(sdr_begin_xn(sdr));
 		findScheme(tokens[2], &vscheme, &elt);
 		if (elt == 0)
 		{
 			printText("Unknown scheme.");
-			return;
+		}
+		else
+		{
+			listEndpointsForScheme(vscheme);
 		}
 
-		listEndpointsForScheme(vscheme);
+		sdr_exit_xn(sdr);
 		break;
 
 	default:
@@ -840,12 +879,15 @@ static void	listProtocols(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
 	for (elt = sdr_list_first(sdr, (getBpConstants())->protocols); elt;
 			elt = sdr_list_next(sdr, elt))
 	{
 		GET_OBJ_POINTER(sdr, ClProtocol, clp, sdr_list_data(sdr, elt));
 		printProtocol(clp);
 	}
+
+	sdr_exit_xn(sdr);
 }
 
 static void	listInductsForProtocol(char *protocolName)
@@ -874,6 +916,7 @@ static void	listInducts(int tokenCount, char **tokens)
 	switch (tokenCount)
 	{
 	case 2:
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sdr_list_first(sdr, (getBpConstants())->protocols);
 				elt; elt = sdr_list_next(sdr, elt))
 		{
@@ -882,17 +925,22 @@ static void	listInducts(int tokenCount, char **tokens)
 			listInductsForProtocol(clpbuf.name);
 		}
 
+		sdr_exit_xn(sdr);
 		break;
 
 	case 3:
+		CHKVOID(sdr_begin_xn(sdr));
 		fetchProtocol(tokens[2], &clpbuf, &elt);
 		if (elt == 0)
 		{
 			printText("Unknown protocol.");
-			return;
+		}
+		else
+		{
+			listInductsForProtocol(clpbuf.name);
 		}
 
-		listInductsForProtocol(clpbuf.name);
+		sdr_exit_xn(sdr);
 		break;
 
 	default:
@@ -926,6 +974,7 @@ static void	listOutducts(int tokenCount, char **tokens)
 	switch (tokenCount)
 	{
 	case 2:
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sdr_list_first(sdr, (getBpConstants())->protocols);
 				elt; elt = sdr_list_next(sdr, elt))
 		{
@@ -934,17 +983,22 @@ static void	listOutducts(int tokenCount, char **tokens)
 			listOutductsForProtocol(clpbuf.name);
 		}
 
+		sdr_exit_xn(sdr);
 		break;
 
 	case 3:
+		CHKVOID(sdr_begin_xn(sdr));
 		fetchProtocol(tokens[2], &clpbuf, &elt);
 		if (elt == 0)
 		{
 			printText("Unknown protocol.");
-			return;
+		}
+		else
+		{
+			listOutductsForProtocol(clpbuf.name);
 		}
 
-		listOutductsForProtocol(clpbuf.name);
+		sdr_exit_xn(sdr);
 		break;
 
 	default:
@@ -1059,7 +1113,7 @@ static void	manageHeapmax(int tokenCount, char **tokens)
 		return;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &bpdb, bpdbObj, sizeof(BpDB));
 	bpdb.maxAcqInHeap = heapmax;
 	sdr_write(sdr, bpdbObj, (char *) &bpdb, sizeof(BpDB));
@@ -1104,6 +1158,23 @@ static void	executeRun(int tokenCount, char **tokens)
 	}
 }
 
+static void	noteWatchValue()
+{
+	BpVdb	*vdb = getBpVdb();
+	Sdr	sdr = getIonsdr();
+	Object	dbObj = getBpDbObject();
+	BpDB	db;
+
+	if (vdb != NULL && dbObj != 0)
+	{
+		CHKVOID(sdr_begin_xn(sdr));
+		sdr_stage(sdr, (char *) &db, dbObj, sizeof(BpDB));
+		db.watching = vdb->watching;
+		sdr_write(sdr, dbObj, (char *) &db, sizeof(BpDB));
+		oK(sdr_end_xn(sdr));
+	}
+}
+
 static void	switchWatch(int tokenCount, char **tokens)
 {
 	BpVdb	*vdb = getBpVdb();
@@ -1234,6 +1305,7 @@ static int	processLine(char *line, int lineLength)
 	char		*tokens[9];
 	struct timeval	done_time;
 	struct timeval	cur_time;
+	char		buffer[80];
 
 	tokenCount = 0;
 	for (cursor = line, i = 0; i < 9; i++)
@@ -1281,6 +1353,12 @@ static int	processLine(char *line, int lineLength)
 		case 'h':
 			printUsage();
 			return 0;
+		case 'v':
+			isprintf(buffer, sizeof buffer,
+					"%s compiled with crypto suite: %s",
+					IONVERSIONNUMBER, crypto_suite_name);
+			printText(buffer);
+			return 0;
 
 		case '1':
 			initializeBp(tokenCount, tokens);
@@ -1412,6 +1490,7 @@ static int	processLine(char *line, int lineLength)
 			if (attachToBp() == 0)
 			{
 				switchWatch(tokenCount, tokens);
+				noteWatchValue();
 			}
 
 			return 0;
