@@ -12,7 +12,9 @@
 #include "cfdpP.h"
 #include "lyst.h"
 
+#ifndef CFDPDEBUG
 #define	CFDPDEBUG	0
+#endif
 
 /*	*	*	Helpful utility functions	*	*	*/
 
@@ -32,6 +34,8 @@ static CfdpDB	*_cfdpConstants()
 {
 	static CfdpDB	buf;
 	static CfdpDB	*db = NULL;
+	Sdr		sdr;
+	Object		dbObject;
 
 	if (db == NULL)
 	{
@@ -40,9 +44,26 @@ static CfdpDB	*_cfdpConstants()
 		 *	as a current database image in later
 		 *	processing.					*/
 
-		sdr_read(getIonsdr(), (char *) &buf, _cfdpdbObject(NULL),
-				sizeof(CfdpDB));
-		db = &buf;
+		sdr = getIonsdr();
+		CHKNULL(sdr);
+		dbObject = _cfdpdbObject(NULL);
+		if (dbObject)
+		{
+			if (sdr_heap_is_halted(sdr))
+			{
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(CfdpDB));
+			}
+			else
+			{
+				CHKNULL(sdr_begin_xn(sdr));
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(CfdpDB));
+				sdr_exit_xn(sdr);
+			}
+
+			db = &buf;
+		}
 	}
 
 	return db;
@@ -127,7 +148,7 @@ int	checkFile(char *fileName)
 	/*	Spawn a separate thread that hangs on opening the file
 	 *	if there's an error in the file system.			*/
 
-	if (pthread_create(&statThread, &attr, checkFileExists, &parms))
+	if (pthread_begin(&statThread, &attr, checkFileExists, &parms))
 	{
 		oK(pthread_mutex_destroy(&mutex));
 		oK(pthread_cond_destroy(&cv));
@@ -162,7 +183,7 @@ int	checkFile(char *fileName)
 
 			/*	Timeout: child stuck, file undefined.	*/
 
-			pthread_cancel(statThread);
+			pthread_end(statThread);
 			parms.fileExists = 0;
 		}
 	}
@@ -308,7 +329,7 @@ static CfdpVdb	*_cfdpvdb(char **name)
 		/*	CFDP volatile database doesn't exist yet.	*/
 
 		sdr = getIonsdr();
-		sdr_begin_xn(sdr);	/*	Just to lock memory.	*/
+		CHKNULL(sdr_begin_xn(sdr));	/*	To lock memory.	*/
 		vdbAddress = psm_zalloc(wm, sizeof(CfdpVdb));
 		if (vdbAddress == 0)
 		{
@@ -374,7 +395,7 @@ int	cfdpInit()
 
 	/*	Recover the CFDP database, creating it if necessary.	*/
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	cfdpdbObject = sdr_find(sdr, _cfdpdbName(), NULL);
 	switch (cfdpdbObject)
 	{
@@ -456,6 +477,64 @@ int	cfdpInit()
 	return 0;		/*	CFDP service is available.	*/
 }
 
+static void	dropVdb(PsmPartition wm, PsmAddress vdbAddress)
+{
+	CfdpVdb		*vdb;
+
+	vdb = (CfdpVdb *) psp(wm, vdbAddress);
+	if (vdb->eventSemaphore != SM_SEM_NONE)
+	{
+		sm_SemDelete(vdb->eventSemaphore);
+	}
+
+	if (vdb->fduSemaphore != SM_SEM_NONE)
+	{
+		sm_SemDelete(vdb->fduSemaphore);
+	}
+
+	if (vdb->currentFile != -1)
+	{
+		close(vdb->currentFile);
+	}
+}
+
+void	cfdpDropVdb()
+{
+	PsmPartition	wm = getIonwm();
+	char		*cfdpvdbName = _cfdpvdbName();
+	PsmAddress	vdbAddress;
+	PsmAddress	elt;
+	char		*stop = NULL;
+
+	if (psm_locate(wm, cfdpvdbName, &vdbAddress, &elt) < 0)
+	{
+		putErrmsg("Failed searching for vdb.", NULL);
+		return;
+	}
+
+	if (elt)
+	{
+		dropVdb(wm, vdbAddress);	/*	Destroy Vdb.	*/
+		psm_free(wm,vdbAddress);
+		if(psm_uncatlg(wm, cfdpvdbName) < 0)
+		{
+			putErrmsg("Failed uncataloging vdb.",NULL);
+		}
+	}
+
+	oK(_cfdpvdb(&stop));			/*	Forget old Vdb.	*/
+}
+
+void	cfdpRaiseVdb()
+{
+	char	*cfdpvdbName = _cfdpvdbName();
+
+	if (_cfdpvdb(&cfdpvdbName) == NULL)	/*	Create new Vdb.	*/
+	{
+		putErrmsg("CFDP can't reinitialize vdb.", NULL);
+	}
+}
+
 Object	getCfdpDbObject()
 {
 	return _cfdpdbObject(NULL);
@@ -482,7 +561,7 @@ int	_cfdpStart(char *utaCmd)
 		return -1;
 	}
 
-	sdr_begin_xn(sdr);	/*	Just to lock memory.		*/
+	CHKERR(sdr_begin_xn(sdr));	/*	Just to lock memory.	*/
 
 	/*	Start the CFDP events clock if necessary.		*/
 
@@ -509,7 +588,7 @@ void	_cfdpStop()		/*	Reverses cfdpStart.		*/
 
 	/*	Tell all CFDP processes to stop.			*/
 
-	sdr_begin_xn(sdr);	/*	Just to lock memory.		*/
+	CHKVOID(sdr_begin_xn(sdr));	/*	Just to lock memory.	*/
 
 	/*	Stop user application input thread.			*/
 
@@ -554,7 +633,7 @@ void	_cfdpStop()		/*	Reverses cfdpStart.		*/
 
 	/*	Now erase all the tasks and reset the semaphores.	*/
 
-	sdr_begin_xn(sdr);	/*	Just to lock memory.		*/
+	CHKVOID(sdr_begin_xn(sdr));	/*	Just to lock memory.	*/
 	cfdpvdb->utaPid = ERROR;
 	cfdpvdb->clockPid = ERROR;
 	if (cfdpvdb->eventSemaphore == SM_SEM_NONE)
@@ -604,7 +683,7 @@ int	cfdpAttach()
 
 	if (cfdpdbObject == 0)
 	{
-		sdr_begin_xn(sdr);	/*	Lock database.		*/
+		CHKERR(sdr_begin_xn(sdr));	/*	Lock database.	*/
 		cfdpdbObject = sdr_find(sdr, _cfdpdbName(), NULL);
 		sdr_exit_xn(sdr);	/*	Unlock database.	*/
 		if (cfdpdbObject == 0)
@@ -632,6 +711,12 @@ int	cfdpAttach()
 	return 0;		/*	CFDP service is available.	*/
 }
 
+void cfdpDetach(){
+	char *stop=NULL;
+	oK(_cfdpvdb(&stop));
+	return;
+}
+
 MetadataList	createMetadataList(Object log)
 {
 	Sdr	sdr = getIonsdr();
@@ -642,7 +727,7 @@ MetadataList	createMetadataList(Object log)
 	 *	in the new list's user data.				*/
 
 	CHKZERO(log);
-	sdr_begin_xn(sdr);
+	CHKZERO(sdr_begin_xn(sdr));
 	list = sdr_list_create(sdr);
 	if (list)
 	{
@@ -822,7 +907,7 @@ int	addFsResp(Object list, CfdpAction action, int status,
 	CHKERR(message == NULL || strlen(secondFileName) < 256);
 	CHKERR(sdr_list_list(sdr, sdr_list_user_data(sdr, list))
 			== cfdpConstants->fsreqLists);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	fsresp.action = action;
 	fsresp.status = status;
 	if (firstFileName)
@@ -922,14 +1007,14 @@ static Object	createInFdu(CfdpTransactionId *transactionId, Entity *entity,
 Object	findInFdu(CfdpTransactionId *transactionId, InFdu *fduBuf,
 		Object *fduElt, int createIfNotFound)
 {
-	unsigned long	sourceEntityId;
-	Sdr		sdr = getIonsdr();
-	CfdpDB		*cfdpConstants = _cfdpConstants();
-	Object		elt;
-	Object		entityObj;
-	Entity		entity;
-	int		foundIt = 0;
-	Object		fduObj;
+	uvast	sourceEntityId;
+	Sdr	sdr = getIonsdr();
+	CfdpDB	*cfdpConstants = _cfdpConstants();
+	Object	elt;
+	Object	entityObj;
+	Entity	entity;
+	int	foundIt = 0;
+	Object	fduObj;
 
 	CHKZERO(transactionId);
 	CHKZERO(fduBuf);
@@ -1331,11 +1416,30 @@ static int	abandonInFdu(CfdpTransactionId *transactionId,
 	return event.reqNbr;
 }
 
+static int	missingFileName(char *fileName, int parmNbr,
+			FilestoreResponse *resp, char *msgBuf, int bufLen)
+{
+	if (fileName)
+	{
+		return 0;
+	}
+	
+	resp->status = 1;
+	isprintf(msgBuf, bufLen, "file name %d not provided", parmNbr);
+	return 1;
+}
+			
 static void	frCreateFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
-	int	fd = iopen(firstFileName, O_CREAT, 0777);
+	int	fd;
 
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
+	fd = iopen(firstFileName, O_CREAT, 0777);
 	if (fd < 0)
 	{
 		resp->status = 1;
@@ -1350,6 +1454,11 @@ static void	frCreateFile(char *firstFileName, char *secondFileName,
 static void	frDeleteFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (unlink(firstFileName) < 0)
 	{
 		resp->status = 1;
@@ -1360,6 +1469,12 @@ static void	frDeleteFile(char *firstFileName, char *secondFileName,
 static void	frRenameFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen)
+	|| missingFileName(secondFileName, 2, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (checkFile(firstFileName) != 1)
 	{
 		resp->status = 1;
@@ -1391,6 +1506,12 @@ static void	frCopyFile(char *firstFileName, char *secondFileName,
 	int	bytesWritten;
 	int	writing = 1;
 
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen)
+	|| missingFileName(secondFileName, 2, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if ((buf = MTAKE(10000)) == NULL)
 	{
 		resp->status = 3;
@@ -1458,6 +1579,12 @@ static void	frCopyFile(char *firstFileName, char *secondFileName,
 static void	frAppendFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen)
+	|| missingFileName(secondFileName, 2, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (checkFile(firstFileName) != 1)
 	{
 		resp->status = 1;
@@ -1479,6 +1606,12 @@ static void	frAppendFile(char *firstFileName, char *secondFileName,
 static void	frReplaceFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen)
+	|| missingFileName(secondFileName, 2, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (checkFile(firstFileName) != 1)
 	{
 		resp->status = 1;
@@ -1500,6 +1633,11 @@ static void	frReplaceFile(char *firstFileName, char *secondFileName,
 static void	frCreateDirectory(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 #if (defined(VXWORKS) || defined(mingw))
 	if (mkdir(firstFileName) < 0)
 #else
@@ -1514,6 +1652,11 @@ static void	frCreateDirectory(char *firstFileName, char *secondFileName,
 static void	frRemoveDirectory(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (rmdir(firstFileName) < 0)
 	{
 		resp->status = 1;
@@ -1524,6 +1667,11 @@ static void	frRemoveDirectory(char *firstFileName, char *secondFileName,
 static void	frDenyFile(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (checkFile(firstFileName) != 1)
 	{
 		return;		/*	Nothing to delete.		*/
@@ -1539,6 +1687,11 @@ static void	frDenyFile(char *firstFileName, char *secondFileName,
 static void	frDenyDirectory(char *firstFileName, char *secondFileName,
 			FilestoreResponse *resp, char *msgBuf, int bufLen)
 {
+	if (missingFileName(firstFileName, 1, resp, msgBuf, bufLen))
+	{
+		return;
+	}
+
 	if (checkFile(firstFileName) != 1)
 	{
 		return;		/*	Nothing to delete.		*/
@@ -2192,14 +2345,14 @@ static Object	selectOutPdu(OutFdu *fdu, int *pduIsFileData)
 
 			sdr_write(sdr, header, (char *) &offset, 4);
 			pdu = zco_create(sdr, ZcoSdrSource, header, 0, 4);
-			if (pdu == 0)
+			if (pdu == (Object) ERROR || pdu == 0)
 			{
 				putErrmsg("No space for file PDU.", NULL);
 				return 0;
 			}
 
 			if (zco_append_extent(sdr, pdu, ZcoFileSource,
-				fdu->fileRef, fdu->progress, length) < 0)
+				fdu->fileRef, fdu->progress, length) <= 0)
 			{
 				putErrmsg("Can't append extent.", NULL);
 				return 0;
@@ -2238,7 +2391,7 @@ int	cfdpDequeueOutboundPdu(Object *pdu, OutFdu *fduBuffer)
 
 	CHKERR(pdu);
 	CHKERR(fduBuffer);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	fduObj = selectOutFdu(fduBuffer);
 	while (fduObj == 0)
 	{
@@ -2259,7 +2412,7 @@ int	cfdpDequeueOutboundPdu(Object *pdu, OutFdu *fduBuffer)
 			return -1;
 		}
 
-		sdr_begin_xn(sdr);
+		CHKERR(sdr_begin_xn(sdr));
 		fduObj = selectOutFdu(fduBuffer);
 	}
 
@@ -2416,9 +2569,9 @@ static int	checkInFduComplete(InFdu *fdu, Object fduObj, Object fduElt)
 
 static int	getFileName(InFdu *fdu, char *stringBuf, int bufLen)
 {
-	Sdr		sdr = getIonsdr();
-	unsigned long	sourceEntityId;
-	unsigned long	transactionNbr;
+	Sdr	sdr = getIonsdr();
+	uvast	sourceEntityId;
+	uvast	transactionNbr;
 
 	if (fdu->workingFileName == 0)
 	{
@@ -2426,7 +2579,8 @@ static int	getFileName(InFdu *fdu, char *stringBuf, int bufLen)
 				&fdu->transactionId.sourceEntityNbr);
 		cfdp_decompress_number(&transactionNbr,
 				&fdu->transactionId.transactionNbr);
-		isprintf(stringBuf, bufLen, "%s%ccfdp.%lu.%lu",
+		isprintf(stringBuf, bufLen,
+				"%s%ccfdp." UVAST_FIELDSPEC "." UVAST_FIELDSPEC,
 				getIonWorkingDirectory(), ION_PATH_DELIMITER,
 				sourceEntityId, transactionNbr);
 		fdu->workingFileName = sdr_string_create(sdr, stringBuf);
@@ -3330,6 +3484,17 @@ static int	handleMetadataPdu(unsigned char *cursor, int bytesRemaining,
 
 	event.fileSize = fileSize;	/*	Projected, not actual.	*/
 	event.messagesToUser = fdu->messagesToUser;
+
+	/*	Must transform the messagesToUser list into a
+	 *	MetadataList for delivery to application.		*/
+
+	sdr_list_user_data_set(sdr, event.messagesToUser,
+		sdr_list_insert_last(sdr, (getCfdpConstants())->usrmsgLists,
+		event.messagesToUser));
+
+	/*	Detach messagesToUser list from FDU so it won't be
+	 *	deleted twice.						*/
+
 	fdu->messagesToUser = 0;
 	if (enqueueCfdpEvent(&event) < 0)
 	{
@@ -3519,7 +3684,7 @@ printf("...wrong CFDP transmission mode...\n");
 
 	/*	Get FDU, creating as necessary.				*/
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	fduObj = findInFdu(&transactionId, &fduBuf, &fduElt, 1);
 	if (fduObj == 0)
 	{
