@@ -17,19 +17,6 @@
 #include "rfx.h"
 #include "lyst.h"
 
-typedef struct
-{
-	time_t	time;
-	long	prevXmitRate;
-	long	xmitRate;
-	int	fromNeighbor;		/*	Boolean.		*/
-} RateChange;
-
-static char	*_cannotForecast()
-{
-	return "Can't complete congestion forecast.";
-}
-
 /*	*	Red-black tree ordering and deletion functions	*	*/
 
 static int	rfx_order_nodes(PsmPartition partition, PsmAddress nodeData,
@@ -248,7 +235,7 @@ int	rfx_system_is_started()
 	return (vdb && vdb->clockPid != ERROR);
 }
 
-IonNeighbor	*findNeighbor(IonVdb *ionvdb, unsigned long nodeNbr,
+IonNeighbor	*findNeighbor(IonVdb *ionvdb, uvast nodeNbr,
 			PsmAddress *nextElt)
 {
 	PsmPartition	ionwm = getIonwm();
@@ -269,7 +256,7 @@ IonNeighbor	*findNeighbor(IonVdb *ionvdb, unsigned long nodeNbr,
 	return NULL;
 }
 
-IonNeighbor	*addNeighbor(IonVdb *ionvdb, unsigned long nodeNbr)
+IonNeighbor	*addNeighbor(IonVdb *ionvdb, uvast nodeNbr)
 {
 	PsmPartition	ionwm = getIonwm();
 	IonNode		*node;
@@ -313,7 +300,7 @@ IonNeighbor	*addNeighbor(IonVdb *ionvdb, unsigned long nodeNbr)
 	return neighbor;
 }
 
-IonNode	*findNode(IonVdb *ionvdb, unsigned long nodeNbr, PsmAddress *nextElt)
+IonNode	*findNode(IonVdb *ionvdb, uvast nodeNbr, PsmAddress *nextElt)
 {
 	PsmPartition	ionwm = getIonwm();
 	IonNode		arg;
@@ -333,7 +320,7 @@ IonNode	*findNode(IonVdb *ionvdb, unsigned long nodeNbr, PsmAddress *nextElt)
 	return NULL;
 }
 
-IonNode	*addNode(IonVdb *ionvdb, unsigned long nodeNbr)
+IonNode	*addNode(IonVdb *ionvdb, uvast nodeNbr)
 {
 	PsmPartition	ionwm = getIonwm();
 	PsmAddress	addr;
@@ -363,7 +350,7 @@ IonNode	*addNode(IonVdb *ionvdb, unsigned long nodeNbr)
 	return node;
 }
 
-int	addSnub(IonNode *node, unsigned long neighborNodeNbr)
+int	addSnub(IonNode *node, uvast neighborNodeNbr)
 {
 	PsmPartition	ionwm = getIonwm();
 	PsmAddress	nextElt;
@@ -425,7 +412,7 @@ int	addSnub(IonNode *node, unsigned long neighborNodeNbr)
 	return 0;
 }
 
-void	removeSnub(IonNode *node, unsigned long neighborNodeNbr)
+void	removeSnub(IonNode *node, uvast neighborNodeNbr)
 {
 	PsmPartition	ionwm = getIonwm();
 	PsmAddress	elt;
@@ -520,673 +507,6 @@ PsmAddress	postProbeEvent(IonNode *node, IonSnub *snub)
 	return sm_list_insert_first(ionwm, ionvdb->probes, addr);
 }
 
-/*	*	RFX congestion forecast functions	*	*	*/
-
-static IonNeighbor	*retrieveNeighbor(unsigned long nodeNbr, Lyst neighbors)
-{
-	LystElt		elt3;
-	IonNeighbor	*np = NULL;
-
-	for (elt3 = lyst_first(neighbors); elt3; elt3 = lyst_next(elt3))
-	{
-		np = (IonNeighbor *) lyst_data(elt3);
-		if (np->nodeNbr == nodeNbr)
-		{
-			break;
-		}
-	}
-
-	if (elt3 == NULL)		/*	This is a new neighbor.	*/
-	{
-		np = (IonNeighbor *) MTAKE(sizeof(IonNeighbor));
-		if (np == NULL)
-		{
-			putErrmsg(_cannotForecast() , NULL);
-			return NULL;
-		}
-
-		memset((char *) np, 0, sizeof(IonNeighbor));
-		np->nodeNbr = nodeNbr;
-		if (lyst_insert_last(neighbors, np) == NULL)
-		{
-			putErrmsg(_cannotForecast() , NULL);
-			return NULL;
-		}
-	}
-
-	return np;
-}
-
-static int	insertRateChange(time_t time, unsigned long xmitRate,
-			int fromNeighbor, unsigned long prevXmitRate,
-			Lyst changes)
-{
-	RateChange	*newChange;
-	LystElt		elt4;
-	RateChange	*change;
-
-	newChange = (RateChange *) MTAKE(sizeof(RateChange));
-	if (newChange == NULL)
-	{
-		putErrmsg(_cannotForecast() , NULL);
-		return -1;			/*	Out of memory.	*/
-	}
-
-	newChange->time = time;
-	newChange->xmitRate = xmitRate;
-	newChange->fromNeighbor = fromNeighbor;
-	newChange->prevXmitRate = prevXmitRate;
-	for (elt4 = lyst_last(changes); elt4; elt4 = lyst_prev(elt4))
-	{
-		change = (RateChange *) lyst_data(elt4);
-		if (change->time <= newChange->time)
-		{
-			break;
-		}
-	}
-
-	if (elt4)
-	{
-		elt4 = lyst_insert_after(elt4, newChange);
-	}
-	else
-	{
-		elt4 = lyst_insert_first(changes, newChange);
-	}
-
-	if (elt4 == NULL)
-	{
-		putErrmsg(_cannotForecast() , NULL);
-		return -1;			/*	Out of memory.	*/
-	}
-
-	return 0;
-}
-
-int	checkForCongestion()
-{
-	time_t		forecastTime;
-	PsmPartition	ionwm;
-	int		ionMemIdx;
-	Lyst		neighbors;
-	Lyst		changes;
-	Sdr		sdr;
-	Object		iondbObj;
-	IonDB		iondb;
-	long		maxForecastOccupancy;
-	long		maxForecastInTransit;
-	long		forecastOccupancy;
-	long		forecastInTransit;
-	long		netGrowthPerSec;
-	IonVdb		*ionvdb;
-	PsmAddress	elt1;
-	IonNeighbor	*neighbor;
-	PsmAddress	elt2;
-	PsmAddress	addr;
-	IonEvent	*event;
-	IonCXref	*contact;
-	LystElt		elt3;
-	IonNeighbor	*np = NULL;
-	LystElt		elt4;
-	RateChange	*change;
-	long		secInEpoch;
-	long		secAdvanced = 0;
-	long		increment;
-	long		spaceRemaining;
-	long		secUntilOutOfSpace;
-	long		netInTransitGrowthPerSec;
-	time_t		alarmTime = 0;
-	long		delta;
-	char		timestampBuffer[TIMESTAMPBUFSZ];
-	char		alarmBuffer[40 + TIMESTAMPBUFSZ];
-	int		result;
-
-	forecastTime = getUTCTime();
-	ionwm = getIonwm();
-	ionMemIdx = getIonMemoryMgr();
-	neighbors = lyst_create_using(ionMemIdx);
-	changes = lyst_create_using(ionMemIdx);
-	if (neighbors == NULL || changes == NULL)
-	{
-		putErrmsg(_cannotForecast() , NULL);
-		return -1;		/*	Out of memory.		*/
-	}
-
-	sdr = getIonsdr();
-	iondbObj = getIonDbObject();
-	sdr_begin_xn(sdr);
-	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
- 	forecastOccupancy = maxForecastOccupancy = iondb.currentOccupancy;
- 	forecastInTransit = maxForecastInTransit = iondb.currentOccupancy;
-	netGrowthPerSec = iondb.productionRate - iondb.consumptionRate;
-	ionvdb = getIonVdb();
-	for (elt1 = sm_rbt_first(ionwm, ionvdb->neighbors); elt1;
-			elt1 = sm_rbt_next(ionwm, elt1))
-	{
-		neighbor = (IonNeighbor*) psp(ionwm, sm_rbt_data(ionwm, elt1));
-		CHKERR(neighbor);
-		netGrowthPerSec += neighbor->recvRate;
-		netGrowthPerSec -= neighbor->xmitRate;
-		np = (IonNeighbor *) MTAKE(sizeof(IonNeighbor));
-		if (np == NULL)
-		{
-			putErrmsg(_cannotForecast() , NULL);
-			sdr_cancel_xn(sdr);
-			return -1;	/*	Out of memory.		*/
-		}
-
-		memcpy((char *) np, (char *) neighbor, sizeof(IonNeighbor));
-		if (lyst_insert_last(neighbors, np) == NULL)
-		{
-			putErrmsg(_cannotForecast() , NULL);
-			sdr_cancel_xn(sdr);
-			return -1;	/*	Out of memory.		*/
-		}
-	}
-
-	/*	Have now got *current* occupancy and growth rate.
-	 *	Next, extract list of all relevant growth rate changes.	*/
-
-	for (elt2 = sm_rbt_first(ionwm, ionvdb->timeline); elt2;
-			elt2 = sm_rbt_next(ionwm, elt2))
-	{
-		addr = sm_rbt_data(ionwm, elt2);
-		event = (IonEvent *) psp(ionwm, addr);
-		if (event->time < forecastTime)
-		{
-			continue;	/*	Happened in the past.	*/
-		}
-
-		/*	Start of transmission?				*/
-
-		if (event->type == IonStartXmit)
-		{
-			contact = (IonCXref *) psp(ionwm, event->ref);
-			if (contact->fromNode == contact->toNode)
-			{
-				/*	This is a loopback contact,
-				 *	which has no net effect on
-				 *	congestion.  Ignore it.		*/
-
-				continue;
-			}
-			
-			if (contact->fromNode != iondb.ownNodeNbr)
-			{
-				continue;	/*	Not relevant.	*/
-			}
-
-			/*	Find affected neighbor; add if nec.	*/
-
-			np = retrieveNeighbor(contact->toNode, neighbors);
-			if (np == NULL)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			if (insertRateChange(event->time, contact->xmitRate,
-					0, np->xmitRate, changes) < 0)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			np->xmitRate = contact->xmitRate;
-			continue;
-		}
-
-		/*	End of transmission?				*/
-
-		if (event->type == IonStopXmit)
-		{
-			contact = (IonCXref *) psp(ionwm, event->ref);
-			if (contact->fromNode == contact->toNode)
-			{
-				/*	This is a loopback contact,
-				 *	which has no net effect on
-				 *	congestion.  Ignore it.		*/
-
-				continue;
-			}
-			
-			if (contact->fromNode != iondb.ownNodeNbr)
-			{
-				continue;	/*	Not relevant.	*/
-			}
-
-			/*	Find affected neighbor; add if nec.	*/
-
-			np = retrieveNeighbor(contact->toNode, neighbors);
-			if (np == NULL)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			if (insertRateChange(event->time, 0,
-					0, np->xmitRate, changes) < 0)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			np->xmitRate = 0;
-			continue;
-		}
-
-		/*	Start of reception?				*/
-
-		if (event->type == IonStartRecv)
-		{
-			contact = (IonCXref *) psp(ionwm, event->ref);
-			if (contact->fromNode == contact->toNode)
-			{
-				/*	This is a loopback contact,
-				 *	which has no net effect on
-				 *	congestion.  Ignore it.		*/
-
-				continue;
-			}
-			
-			if (contact->toNode != iondb.ownNodeNbr)
-			{
-				continue;	/*	Not relevant.	*/
-			}
-
-			/*	Find affected neighbor; add if nec.	*/
-
-			np = retrieveNeighbor(contact->fromNode, neighbors);
-			if (np == NULL)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			if (insertRateChange(event->time, contact->xmitRate,
-					1, np->recvRate, changes) < 0)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			np->recvRate = contact->xmitRate;
-			continue;
-		}
-
-		/*	End of reception?				*/
-
-		if (event->type == IonStopRecv)
-		{
-			contact = (IonCXref *) psp(ionwm, event->ref);
-			if (contact->fromNode == contact->toNode)
-			{
-				/*	This is a loopback contact,
-				 *	which has no net effect on
-				 *	congestion.  Ignore it.		*/
-
-				continue;
-			}
-			
-			if (contact->toNode != iondb.ownNodeNbr)
-			{
-				continue;	/*	Not relevant.	*/
-			}
-
-			/*	Find affected neighbor; add if nec.	*/
-
-			np = retrieveNeighbor(contact->fromNode, neighbors);
-			if (np == NULL)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			if (insertRateChange(event->time, 0,
-					1, np->recvRate, changes) < 0)
-			{
-				sdr_cancel_xn(sdr);
-				return -1;	/*	Out of memory.	*/
-			}
-
-			np->recvRate = 0;
-			continue;
-		}
-	}
-
-	/*	Now revise occupancy level over time as growth occurs
-	 *	and growth rates change.  Occupancy computation
-	 *	optimized by Greg Menke 22 September 2011.		*/
-
-	for (elt4 = lyst_first(changes); elt4; elt4 = lyst_next(elt4))
-	{
-		change = (RateChange *) lyst_data(elt4);
-
-		/*	Let occupancy level change per current rate
-		 *	up to time of next rate change.
-		 *
-		 *	NOTE: change->time can never be less than
-		 *	forecastTime because contacts that started
-		 *	in the past are reflected in the initial
-		 *	netGrowthPerSec; the start of such a contact
-		 *	is never posted to the list of RateChanges. 	*/
-
-		if (change->time < forecastTime)
-		{
-			putErrmsg("Investigate congestion check error.",
-					utoa(change->time));
-			continue;
-		}
-
-		if (iondb.horizon > 0 && iondb.horizon < change->time) 
-		{
-			secInEpoch = iondb.horizon - forecastTime;
-			if (secInEpoch < 0)
-			{
-				secInEpoch = 0;
-			}
-		}
-		else
-		{
-			secInEpoch = change->time - forecastTime;
-		}
-
-		if (netGrowthPerSec > 0)
-		{
-			/*	net growth > 0 means we are receiving
-			 *	more than transmitting.  Project the
-			 *	rate out till the forecast reaches
-			 *	the occupancy ceiling (round up to
-			 *	next second) or we reach the end of
-			 *	the epoch, whichever occurs first.	*/
-
-			spaceRemaining = iondb.occupancyCeiling
-					- forecastOccupancy;
-			secUntilOutOfSpace = 1 + (spaceRemaining
-					/ netGrowthPerSec);
-			if (secInEpoch < secUntilOutOfSpace)
-			{
-				secAdvanced = secInEpoch;
-			}
-			else
-			{
-				secAdvanced = secUntilOutOfSpace;
-			}
-		}
-		else	/*	netGrowthPerSec is <= 0			*/
-		{
-			/*	Note: net growth < 0 means we are
-			 *	transmitting more than receiving.
-			 *	Although this rate of growth is the
-			 *	one that is theoretically correct
-			 *	throughout the epoch, the reduction
-			 *	in occupancy will obviously stop
-			 *	when occupancy reaches zero; from
-			 *	that time until the end of the epoch
-			 *	the theoretical negative net growth
-			 *	in fact represents unused transmission
-			 *	capacity.  We correct for this by
-			 *	changing forecastOccupancy to zero
-			 *	whenever it is driven negative.		*/
-
-			secAdvanced = secInEpoch;
-		}
-
-		increment = netGrowthPerSec * secAdvanced;
-		if (netGrowthPerSec > 0 && increment < 0)
-		{
-			/*	Multiplication overflow.		*/
-
-			forecastOccupancy = iondb.occupancyCeiling;
-		}
-		else
-		{
-			forecastOccupancy += increment;
-		}
-
-		if (forecastOccupancy < 0)
-		{
-			forecastOccupancy = 0;
-		}
-
-		if (forecastOccupancy > maxForecastOccupancy)
-		{
-			maxForecastOccupancy = forecastOccupancy;
-		}
-
-		/*	The in-transit high-water mark is the total
-		 *	occupancy high-water mark less the estimated
-		 *	occupancy due to local bundle origination,
-		 *	i.e., all bundles originating at other nodes
-		 *	that were received at this node and have not
-		 *	yet been either forwarded or delivered.  It
-		 *	constitutes the available margin for local
-		 *	bundle origination and, as such, is the basis
-		 *	for local bundle admission control.		*/
-
-		netInTransitGrowthPerSec = netGrowthPerSec 
-				- iondb.productionRate;
-
-
-
-		increment = netInTransitGrowthPerSec * secAdvanced;
-		if (netInTransitGrowthPerSec > 0 && increment < 0)
-		{
-			/*	Multiplication overflow. Positive Direction*/
-
-			forecastInTransit = iondb.occupancyCeiling;
-		}
-		else
-		{
-			if(netInTransitGrowthPerSec<0 && increment > 0)
-			{
-				/*Multiplication overflow. Negative Direction*/
-				netInTransitGrowthPerSec=0;
-			}
-			else
-			{
-				forecastInTransit += increment;
-			}
-		}
-
-		if (forecastInTransit < 0)
-		{
-			forecastInTransit = 0;
-		}
-
-		if (forecastInTransit > maxForecastInTransit)
-		{
-			maxForecastInTransit = forecastInTransit;
-		}
-#if 0
-		{
-			char msg[1024];
-			sprintf(msg, "c4c1a: horizon=%d, change->time=%d, forecasttime=%d, netgrowthpersec=%d, productionRate=%d",
-					  (int)iondb.horizon,
-					  (int)change->time,
-					  (int)forecastTime,
-					  (int)netGrowthPerSec,
-					  (int)iondb.productionRate );
-
-			writeMemo(msg);
-			sprintf(msg, "c4c1b: forecastOccupancy=%d, forecastInTransit=%d, secAdvanced=%d",
-					  (int)forecastOccupancy,
-					  (int)forecastInTransit,
-					  (int)secAdvanced );
-
-			writeMemo(msg);
-		}
-#endif
-		/*	Advance the forecast time, by epoch or to end,
-		 *	and check for congestion alarm.			*/
-
-		forecastTime += secAdvanced;
-		if (maxForecastOccupancy >= iondb.occupancyCeiling)
-		{
-			alarmTime = forecastTime;
-		}
-
-		if (alarmTime != 0
-		|| (iondb.horizon > 0 && forecastTime > iondb.horizon))
-		{
-			break;		/*	Stop forecast.		*/
-		}
-
-		/*	Apply the adjustment that occurs at the time
-		 *	of this change (end of prior epoch).		*/
-
-		delta = change->xmitRate - change->prevXmitRate;
-		if (change->fromNeighbor)
-		{
-			netGrowthPerSec += delta;
-		}
-		else
-		{
-			netGrowthPerSec -= delta;
-		}
-	}
-
-	/*	Have determined final net growth rate as of end of
-	 *	last scheduled contact.					*/
-
-	if (netGrowthPerSec > 0 && alarmTime == 0)
-	{
-		/*	Unconstrained growth; will max out eventually,
-		 *	just need to determine when.  Final epoch.	*/
-
-		if (iondb.horizon > 0) 
-		{
-			secInEpoch = (iondb.horizon - forecastTime);
-		}
-		else
-		{
-			secInEpoch = LONG_MAX;
-		}
-
-		/*	net growth > 0 means we are receiving more than
-		 *	transmitting.  Project the rate out till the
-		 *	forecast reaches the occupancy ceiling (round
-		 *	up to the next second) or we reach the end of
-		 *	the epoch, whichever occurs first.		*/
-
-		spaceRemaining = iondb.occupancyCeiling - forecastOccupancy;
-		secUntilOutOfSpace = 1 + (spaceRemaining / netGrowthPerSec);
-		if (secInEpoch < secUntilOutOfSpace)
-		{
-			secAdvanced = secInEpoch;
-		}
-		else
-		{
-			secAdvanced = secUntilOutOfSpace;
-		}
-
-		increment = netGrowthPerSec * secAdvanced;
-		if (increment < 0)
-		{
-			/*	Multiplication overflow.		*/
-
-			forecastOccupancy = iondb.occupancyCeiling;
-		}
-		else
-		{
-			forecastOccupancy += increment;
-		}
-
-		if (forecastOccupancy > maxForecastOccupancy)
-		{
-			maxForecastOccupancy = forecastOccupancy;
-		}
-
-		netInTransitGrowthPerSec = netGrowthPerSec 
-				- iondb.productionRate;
-		increment = netInTransitGrowthPerSec * secAdvanced;
-		if (netInTransitGrowthPerSec > 0 && increment < 0)
-		{
-			/*	Multiplication overflow.		*/
-
-			forecastInTransit = iondb.occupancyCeiling;
-		}
-		else
-		{
-			forecastInTransit += increment;
-		}
-
-		if (forecastInTransit < 0)
-		{
-			forecastInTransit = 0;
-		}
-
-		if (forecastInTransit > maxForecastInTransit)
-		{
-			maxForecastInTransit = forecastInTransit;
-		}
-
-		forecastTime += secAdvanced;
-		if (maxForecastOccupancy >= iondb.occupancyCeiling)
-		{
-			alarmTime = forecastTime;
-		}
-	}
-
-	if (alarmTime == 0)
-	{
-		writeMemo("[i] No congestion collapse predicted.");
-	}
-	else
-	{
-		/*	Have determined time at which occupancy limit
-		 *	will be reached.				*/
-
-		writeTimestampUTC(alarmTime, timestampBuffer);
-		isprintf(alarmBuffer, sizeof alarmBuffer,
-				"[i] Congestion collapse forecast: %s.",
-				timestampBuffer);
-		writeMemo(alarmBuffer);
-		if (iondb.alarmScript)
-		{
-			sdr_string_read(sdr, alarmBuffer, iondb.alarmScript);
-			pseudoshell(alarmBuffer);
-		}
-	}
-
-	/*	In any case, update maxForecastOccupancy and InTransit.	*/
-
-	iondb.maxForecastOccupancy = maxForecastOccupancy;
-	iondb.maxForecastInTransit = maxForecastInTransit;
-	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
-	result = sdr_end_xn(sdr);
-
-	/*	Release memory used for neighbors and changes lists.	*/
-
-	elt3 = lyst_first(neighbors);
-	while (elt3)
-	{
-		np = (IonNeighbor *) lyst_data(elt3);
-		MRELEASE(np);
-		lyst_delete(elt3);
-		elt3 = lyst_first(neighbors);
-	}
-
-	lyst_destroy(neighbors);
-	elt4 = lyst_first(changes);
-	while (elt4)
-	{
-		change = (RateChange *) lyst_data(elt4);
-		MRELEASE(change);
-		lyst_delete(elt4);
-		elt4 = lyst_first(changes);
-	}
-
-	lyst_destroy(changes);
-	if (result < 0)
-	{
-		putErrmsg("Failed on check for congestion.", NULL);
-		return -1;
-	}
-
-	return 0;
-}
-
 /*	*	RFX contact list management functions	*	*	*/
 
 static PsmAddress	insertCXref(IonCXref *cxref)
@@ -1382,8 +702,7 @@ static PsmAddress	insertCXref(IonCXref *cxref)
 }
 
 PsmAddress	rfx_insert_contact(time_t fromTime, time_t toTime,
-			unsigned long fromNode, unsigned long toNode,
-			unsigned long xmitRate)
+			uvast fromNode, uvast toNode, unsigned int xmitRate)
 {
 	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
@@ -1405,7 +724,7 @@ PsmAddress	rfx_insert_contact(time_t fromTime, time_t toTime,
 	CHKZERO(toTime > fromTime);
 	CHKZERO(fromNode);
 	CHKZERO(toNode);
-	sdr_begin_xn(sdr);
+	CHKZERO(sdr_begin_xn(sdr));
 
 	/*	Make sure contact doesn't overlap with any pre-existing
 	 *	contacts.						*/
@@ -1645,8 +964,7 @@ static void	deleteContact(PsmAddress cxaddr)
 			rfx_erase_data, NULL);
 }
 
-int	rfx_remove_contact(time_t fromTime, unsigned long fromNode,
-		unsigned long toNode)
+int	rfx_remove_contact(time_t fromTime, uvast fromNode, uvast toNode)
 {
 	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
@@ -1665,7 +983,7 @@ int	rfx_remove_contact(time_t fromTime, unsigned long fromNode,
 	memset((char *) &arg, 0, sizeof(IonCXref));
 	arg.fromNode = fromNode;
 	arg.toNode = toNode;
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (fromTime)		/*	Not a wild-card deletion.	*/
 	{
 		arg.fromTime = fromTime;
@@ -1897,8 +1215,8 @@ static int	insertRXref(IonRXref *rxref)
 	return rxaddr;
 }
 
-Object	rfx_insert_range(time_t fromTime, time_t toTime, unsigned long fromNode,
-		unsigned long toNode, unsigned int owlt)
+Object	rfx_insert_range(time_t fromTime, time_t toTime, uvast fromNode,
+		uvast toNode, unsigned int owlt)
 {
 	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
@@ -1941,7 +1259,7 @@ Object	rfx_insert_range(time_t fromTime, time_t toTime, unsigned long fromNode,
 	CHKZERO(toTime > fromTime);
 	CHKZERO(fromNode);
 	CHKZERO(toNode);
-	sdr_begin_xn(sdr);
+	CHKZERO(sdr_begin_xn(sdr));
 
 	/*	Make sure range doesn't overlap with any pre-existing
 	 *	ranges.							*/
@@ -2185,8 +1503,7 @@ static void	deleteRange(PsmAddress rxaddr, int conditional)
 			rfx_erase_data, NULL);
 }
 
-int	rfx_remove_range(time_t fromTime, unsigned long fromNode,
-		unsigned long toNode)
+int	rfx_remove_range(time_t fromTime, uvast fromNode, uvast toNode)
 {
 	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
@@ -2205,7 +1522,7 @@ int	rfx_remove_range(time_t fromTime, unsigned long fromNode,
 	memset((char *) &arg, 0, sizeof(IonRXref));
 	arg.fromNode = fromNode;
 	arg.toNode = toNode;
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (fromTime)		/*	Not a wild-card deletion.	*/
 	{
 		arg.fromTime = fromTime;
@@ -2358,7 +1675,7 @@ static int	loadContact(Object elt)
 	cxref.xmitRate = contact.xmitRate;
 	cxref.contactElt = elt;
 	cxref.routingObject = 0;
-	if (insertCXref(&cxref) < 0)
+	if (insertCXref(&cxref) == 0)
 	{
 		return -1;
 	}
@@ -2368,6 +1685,7 @@ static int	loadContact(Object elt)
 
 int	rfx_start()
 {
+	PsmPartition	ionwm = getIonwm();
 	Sdr		sdr = getIonsdr();
 	IonVdb		*vdb = getIonVdb();
 	Object		iondbObj;
@@ -2375,9 +1693,17 @@ int	rfx_start()
 	Object		elt;
 
 	iondbObj = getIonDbObject();
-	sdr_begin_xn(sdr);	/*	Just to lock memory.		*/
+	CHKERR(sdr_begin_xn(sdr));	/*	To lock memory.		*/
 	sdr_read(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 
+	/* Destroy and re-create volatile contact and range databases.
+	 * This prevents contact/range duplication as a result of adds
+	 * before starting ION. */
+	sm_rbt_destroy(ionwm, vdb->contactIndex, rfx_erase_data, NULL);
+	sm_rbt_destroy(ionwm, vdb->rangeIndex, rfx_erase_data, NULL);
+	vdb->contactIndex = sm_rbt_create(ionwm);
+	vdb->rangeIndex = sm_rbt_create(ionwm);
+
 	/*	Load range index for all asserted ranges.  In so
 	 *	doing, load the nodes for which ranges are known
 	 *	and load events for all predicted changes in range.	*/
@@ -2427,6 +1753,12 @@ void	rfx_stop()
 	PsmPartition	ionwm = getIonwm();
 	IonVdb		*vdb = getIonVdb();
 
+	/*	Clear ZCO availability information.			*/
+
+	sm_SemEnd(vdb->zcoSemaphore);
+	vdb->zcoClaimants = 0;
+	vdb->zcoClaims = 0;
+
 	/*	Stop the rfx clock if necessary.			*/
 
 	if (vdb->clockPid != ERROR)
